import {Component, OnDestroy, OnInit} from '@angular/core';
import {TabuMiddlewareService} from '../dao/TabuMiddlewareService';
import {GameStatus} from '../interface/gameStatus';
import {ActivatedRoute, Router} from '@angular/router';
import {IsAllowedToPlay} from '../dao/isAllowedToPlay';
import {SocketDataService} from '../dao/socketDataService';
import {Subject} from 'rxjs';
import {takeUntil} from 'rxjs/operators';
@Component({
  selector: 'app-guesser',
  templateUrl: './guesser.component.html',
  styleUrls: ['./guesser.component.scss']
})
export class GuesserComponent implements OnInit, OnDestroy {

  private ngUnsubscribe = new Subject();
  subNewCard: any;
  subEndRound: any;
  sessionName = '';
  TIME_LIMIT = 100;
  timePassed = 0;
  timeRemaining = 60;
  timerInterval = 0;
  timeLeft = this.TIME_LIMIT;
  gameStatus: GameStatus = {
    sessionID: -1,
    red: 0,
    blue: 0,
    redTurn: true,
    activeExplainer: 1,
    activeCard: 1
  };
  team = '';

  constructor(private router: Router,
              private activatedRoute: ActivatedRoute,
              private service: TabuMiddlewareService,
              private isAllowedToPlay: IsAllowedToPlay,
              private socketDataService: SocketDataService) { }

  ngOnInit(): void {
    this.activatedRoute.paramMap.subscribe(params => {
      this.sessionName = String(params.get('sessionName')).toLowerCase();
      this.team = String(params.get('team'));
      if (!this.isAllowedToPlay.isAllowed || this.isAllowedToPlay.role !== 'guesser') {
        this.router.navigate([this.sessionName + '/' + this.team]);
      }
      this.fillGamestatus();
      this.getSocketData();
    });
    this.startTimer();
  }

  ngOnDestroy(): void {
    this.unsubscribeAll();
  }

  getSocketData(): void {
    this.subNewCard = this.socketDataService.getNewCard(this.sessionName)
      .pipe(takeUntil(this.ngUnsubscribe))
      .subscribe(data => {
        console.log('GUESSER SOCKET CARD: ', data);
        this.fillGamestatus();
      });
    this.subEndRound = this.socketDataService.getEndRound(this.sessionName)
      .pipe(takeUntil(this.ngUnsubscribe))
      .subscribe(data => {
        console.log('GUESSER SOCKET END ROUND: ', data);
        if (JSON.parse(data)) {
          this.unsubscribeAll();
          this.router.navigate([this.sessionName + '/' + this.team]);
        }
      });
  }

  onTimesUp(): void{
    this.unsubscribeAll();
    this.router.navigate([this.sessionName + '/' + this.team]);
  }

  startTimer(): void {
    this.startCountdown();
    this.timerInterval = setInterval(() => {
      this.timePassed = this.timePassed += 1;
      this.timeLeft = (this.TIME_LIMIT - this.timePassed);
      if (this.timeLeft === 0) {
        console.log('Times up!');
        //this.onTimesUp();
      }
    }, 593);
  }
  startCountdown(): void {
    setInterval(() => {
      this.timeRemaining = this.timeRemaining -= 1;
    }, 1000);
  }

  fillGamestatus(): void {
    this.service.getGamestatus({spielname: this.sessionName}).then(value => {
      const status = JSON.parse(value.status);
      if (status) {
        this.gameStatus.sessionID = JSON.parse(value.sessionID);
        this.gameStatus.red = JSON.parse(value.red);
        this.gameStatus.blue = JSON.parse(value.blue);
        this.gameStatus.redTurn = JSON.parse(value.redTurn);
        this.gameStatus.activeExplainer = JSON.parse(value.activeExplainer);
        this.gameStatus.activeCard = JSON.parse(value.activeCard);
      }
    });
  }

  unsubscribeAll(): void {
    this.subNewCard.unsubscribe();
    this.subEndRound.unsubscribe();

    this.ngUnsubscribe.next();
    this.ngUnsubscribe.complete();
  }
}
