import {Component, OnDestroy, OnInit} from '@angular/core';
import {ActivatedRoute, Router} from '@angular/router';
import {TabuMiddlewareService} from '../dao/TabuMiddlewareService';
import {GameStatus} from '../interface/gameStatus';
import {IsAllowedToPlay} from '../dao/isAllowedToPlay';
import {SocketDataService} from '../dao/socketDataService';
import {Subject} from 'rxjs';
import {takeUntil} from 'rxjs/operators';

@Component({
  selector: 'app-overview',
  templateUrl: './overview.component.html',
  styleUrls: ['./overview.component.scss']
})
export class OverviewComponent implements OnInit, OnDestroy {

  private ngUnsubscribe = new Subject();
  subNewGame: any;
  subNewCard: any;
  subNewRound: any;
  subEndRound: any;
  wantToBeExplainer = false;
  isActiveRound = false;
  red = 'choose';
  blue = 'choose';
  nextTeam = 'Blau';
  team = '';
  buttonNewGameDisabled = true;
  buttonNextRoundDisabled = true;
  sessionName = '';
  membership = 'Wähle dein Team';
  gameStatus: GameStatus = {
    sessionID: -1,
    red: 0,
    blue: 0,
    redTurn: true,
    activeExplainer: 1,
    activeCard: 1
  };
  constructor(private router: Router,
              private activatedRoute: ActivatedRoute,
              private service: TabuMiddlewareService,
              private isAllowedToPlay: IsAllowedToPlay,
              private socketDataService: SocketDataService) { }

  ngOnInit(): void {
    this.activatedRoute.paramMap.subscribe(params => {
      this.sessionName = String(params.get('sessionName')).toLowerCase();
      this.team = String(params.get('team'));
      this.getSocketData();
      this.service.isSession({spielname: this.sessionName}).then(value => {
        if (!JSON.parse(value.status)) {
          this.router.navigate(['error']);
        } else {
          console.log('Team:', this.team);
          if (this.team === 'red'){
            this.membership = 'Du gehörst zu Team rot!';
            this.buttonNewGameDisabled = false;
            this.red = 'red';
            this.blue = 'choose';
          }
          else if (this.team === 'blue'){
            this.membership = 'Du gehörst zu Team blau!';
            this.buttonNewGameDisabled = false;
            this.blue = 'blue';
            this.red = 'choose';
          }
          else if (this.team === 'null'){
            this.membership = 'Wähle dein Team';
          }
          else {
            this.router.navigate(['error']);
            return;
          }
          console.log("GET SOCKET DATA");
          this.getGameStatus();
        }
      });
    });
  }

  ngOnDestroy(): void {
    this.unsubscribeAll();
  }

  getSocketData(): void {
    this.subNewGame = this.socketDataService.getNewGame (this.sessionName)
      .pipe(takeUntil(this.ngUnsubscribe)).subscribe(data => {
        console.log('OVERVIEW SOCKET GAME: ', data);
        this.getGameStatus();
      });
    this.subNewCard = this.socketDataService.getNewCard(this.sessionName)
      .pipe(takeUntil(this.ngUnsubscribe)).subscribe(data => {
        console.log('OVERVIEW SOCKET CARD: ', data);
        this.getGameStatus();
      });
    this.subEndRound = this.socketDataService.getEndRound(this.sessionName)
      .pipe(takeUntil(this.ngUnsubscribe)).subscribe(data => {
        console.log('OVERVIEW SOCKET END ROUND: ', data);
        if (JSON.parse(data)) {
          this.getGameStatus();
        }
      });
    this.subNewRound = this.socketDataService.getNewRound(this.sessionName)
      .pipe(takeUntil(this.ngUnsubscribe)).subscribe(data => {
        console.log('OVERVIEW SOCKET NEW ROUND: ', data);
        if (JSON.parse(data)) {
          console.log('OVERVIEW: ', data);
          if (((this.team === 'red' && this.nextTeam === 'Rot') || (this.team === 'blue' && this.nextTeam === 'Blau'))
            && !this.wantToBeExplainer) {
            console.log('OVERVIEW GO GUESSER: ', data);
            this.isAllowedToPlay.isAllowed = true;
            this.isAllowedToPlay.role = 'guesser';
            this.unsubscribeAll();
            this.router.navigate([this.sessionName + '/' + this.team + '/guesser']);
          } else if ((this.team === 'red' || this.team === 'blue') && !this.wantToBeExplainer) {
            console.log('OVERVIEW GO WATCHDOG: ', data);
            this.isAllowedToPlay.isAllowed = true;
            this.isAllowedToPlay.role = 'watchdog';
            this.unsubscribeAll();
            this.router.navigate([this.sessionName + '/' + this.team + '/watchdog']);
          }
        }
      });
  }

  startGame(): void {
    if (this.team === 'red' || this.team === 'blue') {
      this.service.newGame({spielname: this.sessionName}).then(value => {
        const status = JSON.parse(value.status);
        if (status) {
          this.getGameStatus();
        }
      }).catch(reason => console.log(reason));
    }
  }

  nextRound(): void {
    this.service.getS2C({spielname: this.sessionName}).then(value => {
      console.log('START NEXT ROUND', value);
      this.service.newRound({spielname: this.sessionName});
      this.isAllowedToPlay.isAllowed = true;
      this.isAllowedToPlay.role = 'explainer';
      this.unsubscribeAll();
      this.router.navigate([this.router.url + '/explainer', ]);
      this.wantToBeExplainer = true;
    });
  }

  joinTeam(team: string): void {
    this.team = team;
    this.unsubscribeAll();
    this.router.navigate([this.sessionName + '/' + team]);
  }

  getGameStatus(): void {
    this.service.getGamestatus({spielname: this.sessionName}).then(value => {
      const status = JSON.parse(value.status);
      if (status) {
        this.gameStatus.sessionID = JSON.parse(value.sessionID);
        this.gameStatus.red = JSON.parse(value.red);
        this.gameStatus.blue = JSON.parse(value.blue);
        this.gameStatus.redTurn = JSON.parse(value.redTurn);
        this.gameStatus.activeExplainer = JSON.parse(value.activeExplainer);
        this.gameStatus.activeCard = JSON.parse(value.activeCard);
        if (this.gameStatus.redTurn){
          this.nextTeam = 'Rot';
          this.buttonNextRoundDisabled = this.team !== 'red';
        }
        else{
          this.nextTeam = 'Blau';
          this.buttonNextRoundDisabled = this.team !== 'blue';
        }
        this.isActiveRound = this.gameStatus.activeExplainer === 1;
      }
      console.log(this.gameStatus);
    }).catch(reason => console.log(reason));
  }

  unsubscribeAll(): void {
    this.subNewCard.unsubscribe();
    this.subNewRound.unsubscribe();
    this.subEndRound.unsubscribe();
    this.subNewGame.unsubscribe();

    this.ngUnsubscribe.next();
    this.ngUnsubscribe.complete();
  }
}
