import { Component, OnInit } from '@angular/core';
import {Router} from '@angular/router';
import {TabuMiddlewareService} from '../dao/TabuMiddlewareService';
import {SocketDataService} from '../dao/socketDataService';

@Component({
  selector: 'app-select-game',
  templateUrl: './select-game.component.html',
  styleUrls: ['./select-game.component.scss']
})
export class SelectGameComponent implements OnInit {

  buttonCreateGameDisabled = true;
  buttonJoinGameDisabled = true;
  constructor(private router: Router,
              private service: TabuMiddlewareService,
              private socketDataService: SocketDataService) { }

  ngOnInit(): void {
  }

  getSpielSessions(): any {
    console.log('Start');
    const response = this.service.getSessions();
    console.log('Response:', response);
  }

  createGame(sessionName: string): void {
    sessionName = sessionName.toLowerCase();
    this.service.addSession({spielname: sessionName}).then(value => {
      console.log(value);
      const status = JSON.parse(value.status);
      if (status) {
        this.socketDataService.connect(sessionName);
        this.router.navigate([sessionName]);
      }
    }).catch(reason => {
      console.log(reason);
      console.log('Es ist ein Fehler aufgetreten beim Erstellen eines neuen Spiels');
    });
  }

  joinGame(sessionName: string): void {
    this.socketDataService.connect(sessionName);
    this.router.navigate([sessionName]);
  }

  keyPressAlphaNumeric(event: KeyboardEvent): boolean {
    if (/[a-zA-Z0-9]/.test(event.key)) {
      return true;
    } else {
      event.preventDefault();
      return false;
    }
  }

  enter(sessionName: string): void{
    sessionName = sessionName.toLowerCase();
    if (!this.buttonCreateGameDisabled){
      this.createGame(sessionName);
    }
    if (!this.buttonJoinGameDisabled){
      this.joinGame(sessionName);
    }
  }

  checkSpielname(sessionName: string): void {
    sessionName = sessionName.toLowerCase();
    if (sessionName.length === 0) {
      this.buttonCreateGameDisabled = true;
      this.buttonJoinGameDisabled = true;
      return;
    } else {
      this.service.isSession({'spielname': sessionName}).then(value => {
        const status = JSON.parse(value.status);
        if (status) {
          this.buttonCreateGameDisabled = true;
          this.buttonJoinGameDisabled = false;
        } else {
          this.buttonCreateGameDisabled = false;
          this.buttonJoinGameDisabled = true;
        }
      });
    }
  }
}
